
#ifndef _PPROCESS_H_
#define _PPROCESS_H_

/**
 * @file pprocess.h
 *
 * Hlavickovy soubor k modulu pro praci s chranenymi procesy
 */

#include <ntifs.h>



// Popis vyznamu jednotlivych funkci naleznete v komentarich u jejich 
// implementace v souboru pprocess.c
NTSTATUS ProcessProtect(HANDLE ProcessId);
NTSTATUS ProcessUnprotect(HANDLE process_id);
NTSTATUS ProcessIsProtected(HANDLE ProcessId, PBOOLEAN Result);


NTSTATUS PProcessInit(VOID);
VOID PProcessFinit(VOID);



#endif
